//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "JsonViewerUnit.h"
#include "display\ValueStringFormatter.h"
#include "device\MiniscopeDeviceInterface.h"
#include "Log.h"

#include <json/json.h>
#include <fstream>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmJsonViewer *frmJsonViewer;
//---------------------------------------------------------------------------
__fastcall TfrmJsonViewer::TfrmJsonViewer(TComponent* Owner)
	: TForm(Owner),
	callbackClose(NULL)
{
}
//---------------------------------------------------------------------------

int TfrmJsonViewer::LoadFile(AnsiString filename)
{
	std::ifstream ifs(filename.c_str());
	if (!ifs.is_open())
	{
		LOG(E_LOG_TRACE, "Failed to open %s", filename.c_str());
		return -1;
	}
	std::string strJson((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
	ifs.close();

	Json::Value root;   // will contains the root value after parsing.
	Json::Reader reader;
	bool parsingSuccessful = reader.parse( strJson, root );
	if ( !parsingSuccessful )
	{
		LOG(E_LOG_TRACE, "Failed to parse JSON file %s", filename.c_str());
		return -2;
	}

	struct JsonData tmp;
	tmp.fDivX = root.get("fDivX", 0).asDouble();
	if (tmp.fDivX <= 0)
	{
		LOG(E_LOG_TRACE, "Incorrect fDivX value");
		return -3;
	}

	tmp.iCouplingType = root.get("Coupling", -1).asInt();
	if (tmp.iCouplingType < 0)
	{
		LOG(E_LOG_TRACE, "Incorrect coupling type value");
		return -4;
	}

	tmp.iTriggerType = root.get("TriggerType", -1).asInt();
	if (tmp.iTriggerType < 0)
	{
		LOG(E_LOG_TRACE, "Incorrect trigger type value");
		return -4;
	}

	tmp.asTimestamp = root.get("Timestamp", "").asString().c_str();

	const Json::Value samples = root["Data"];
	if (samples.size() == 0)
	{
		LOG(E_LOG_TRACE, "No data found in JSON file");
		return -5;
	}

	// now we are sure that JSON file is valid
	// copy temporary data
	data = tmp;
	data.samples.clear();
	data.samples.reserve(samples.size());

    edFilename->Text = filename;
	for ( unsigned int index = 0; index < samples.size(); ++index )  // Iterates over the sequence elements.
	{
		data.samples.push_back( samples[index].asDouble() );
	}

	float fSamplVal = (float)1.0 / data.fDivX;
	lblSamplingVal->Caption =
		ValueStringFormatter::FreqToStr(fSamplVal, 1).c_str();

	lblTriggerTypeVal->Caption = DeviceInterface::GetTriggerTypeName((E_TRIGGER_TYPE)data.iTriggerType);
	lblCouplingTypeVal->Caption = DeviceInterface::GetCouplingTypeName((E_COUPLING_TYPE)data.iCouplingType);
	lblTimestampVal->Caption = data.asTimestamp;
	lblSamplesCountVal->Caption = data.samples.size();

	return 0;
}

void __fastcall TfrmJsonViewer::FormClose(TObject *Sender, TCloseAction &Action)
{
	if (callbackClose)
	{
		callbackClose();
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmJsonViewer::btnExitViewerClick(TObject *Sender)
{
	Close();	
}
//---------------------------------------------------------------------------

