/** \file Main.h
    \brief Application main form header file
*/

#ifndef MainH
#define MainH
//---------------------------------------------------------------------------
#include "display/Display.h"
#include "DataFrame.h"
#include "Mutex.h"
#include <ActnList.hpp>
#include <Classes.hpp>
#include <ComCtrls.hpp>
#include <Controls.hpp>
#include <Dialogs.hpp>
#include <ExtCtrls.hpp>
#include <ImgList.hpp>
#include <Menus.hpp>
#include <StdCtrls.hpp>
#include <ToolWin.hpp>
#include <StdActns.hpp>
#include <memory>


class DeviceInterface;
struct S_SCOPE_CAPABILITIES;

//---------------------------------------------------------------------------
/** \brief Main application window
*/
class TfrmMain : public TForm
{
__published:	// IDE-managed Components
	TPanel *pnlCenter;
	TTimer *Timer;
	TStatusBar *StatusBar;
	TMainMenu *Menu;
	TMenuItem *File1;
	TMenuItem *miHelp;
	TMenuItem *E1;
	TMenuItem *miSettings;
	TImageList *imglToolbar;
	TCoolBar *CoolBar;
	TToolBar *ToolBarMode;
	TToolButton *btnModeScope;
	TToolButton *btnModeSpectrum;
	TToolButton *btnModeRecorder;
	TToolBar *ToolBarMouse;
	TToolButton *btnMouseCursor;
	TToolButton *btnMouseZoomXY;
	TToolButton *btnMouseMove;
	TActionList *ActionList;
	TAction *ActionChangeMouseModeCursor;
	TAction *ActionChangeMouseModeOffset;
	TAction *ActionChangeMouseModeZoom;
	TToolButton *btnMouseZoomX;
	TToolButton *btnMouseZoomY;
	TAction *ActionChangeMouseModeZoomX;
	TAction *ActionChangeMouseModeZoomY;
	TToolButton *btnZoomClear;
	TAction *ActionZoomClear;
	TAction *ActionModeScope;
	TAction *ActionModeSpectrum;
	TMenuItem *miAbout;
	TMenuItem *miCommonSettings;
	TMenuItem *miDeviceSettings;
	TMenuItem *miView;
	TMenuItem *miViewLog;
	TAction *ActionAcquireStart;
	TAction *ActionAcquireStop;
	TTimer *tmrUpdateConnState;
	TPaintBox *PaintBoxCenter;
	TAction *ActionCopyPlotClipbrd;
	TMenuItem *miCopyPlotToClipbrd;
	TPanel *pnlZoomInfo;
	TLabel *lblZoomX;
	TLabel *lblZoomY;
	TMenuItem *miTools;
	TMenuItem *miStatistics;
	TAction *ActionShowCommonSettings;
	TAction *ActionModeRecorder;
	TTimer *tmrRecorder;
	TMenuItem *miExportPlotBitmap;
	TAction *ActionExportPlotBitmap;
	TSaveDialog *SaveDialog;
	TAction *ActionCalibrateSensitivity;
	TMenuItem *miCalibrateSensitivity;
	TAction *ActionExportData;
	TMenuItem *miExportcurrentdatatoJSONfile;
	TToolButton *ToolButton1;
	TToolButton *ToolButton2;
	TToolButton *btinAutoFitY;
	TAction *ActionAutoFitY;
	TAction *ActionAutoFitX;
	TToolButton *btnAutoFitX;
	TMenuItem *miLoadJsonFile;
	TAction *ActionLoadJsonFile;
	TPanel *pnlRight;
	TPanel *pnlOscControl;
	TLabel *lblSensitivity;
	TLabel *lblSensitivityVal;
	TLabel *lblTriggerType;
	TLabel *lblBuffer;
	TLabel *lblSampling;
	TLabel *lblSamplingVal;
	TLabel *lblTriggerLevel;
	TLabel *lblTriggerLevelVal;
	TLabel *lblCouplingType;
	TLabel *lblInterval;
	TLabel *lblSeconds;
	TTrackBar *trbarSensitivity;
	TComboBox *cbTriggerType;
	TButton *btnManualTrigger;
	TComboBox *cbBufferSize;
	TTrackBar *trbarSampling;
	TTrackBar *trbarTriggerLevel;
	TComboBox *cbCouplingType;
	TButton *btnStart;
	TButton *btnStop;
	TEdit *edInterval;
	TButton *btnRecorderStart;
	TButton *btnRecorderStop;
	TButton *btnRecorderClear;
	TProgressBar *ProgressBarFrameRx;
	TFileExit *FileExit1;
	TMenuItem *miExit;
	TMenuItem *N1;
	TOpenDialog *OpenDialog;
	TAction *ActionShowDataFilterChain;
	TMenuItem *Datafilterchain1;
	TAction *ActionMs3Record;
	TMenuItem *RecorddatatoMS3file1;
	TMenuItem *N2;
	TMenuItem *N3;
	TAction *ActionMs3Load;
	TMenuItem *LoadMS3file1;
	TButton *btnRecordingIntervalApply;
	TMenuItem *miMs3FileRepair;
	TAction *ActionMs3FileRepair;
	void __fastcall pnlCenterResize(TObject *Sender);
	void __fastcall TimerTimer(TObject *Sender);
	void __fastcall pnlCenterMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall ActionChangeMouseModeCursorExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeOffsetExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomXExecute(TObject *Sender);
	void __fastcall ActionChangeMouseModeZoomYExecute(TObject *Sender);
	void __fastcall ActionZoomClearExecute(TObject *Sender);
	void __fastcall pnlCenterMouseUp(TObject *Sender, TMouseButton Button,
		  TShiftState Shift, int X, int Y);
	void __fastcall ActionModeScopeExecute(TObject *Sender);
	void __fastcall ActionModeSpectrumExecute(TObject *Sender);
	void __fastcall miAboutClick(TObject *Sender);
	void __fastcall FormCreate(TObject *Sender);
	void __fastcall miDeviceSettingsClick(TObject *Sender);
	void __fastcall miViewLogClick(TObject *Sender);
	void __fastcall ActionAcquireStartExecute(TObject *Sender);
	void __fastcall ActionAcquireStopExecute(TObject *Sender);
	void __fastcall tmrUpdateConnStateTimer(TObject *Sender);
	void __fastcall cbTriggerTypeChange(TObject *Sender);
	void __fastcall btnManualTriggerClick(TObject *Sender);
	void __fastcall cbBufferSizeChange(TObject *Sender);
	void __fastcall cbCouplingTypeChange(TObject *Sender);
	void __fastcall PaintBoxCenterPaint(TObject *Sender);
	void __fastcall PaintBoxCenterMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall PaintBoxCenterMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
	void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
	void __fastcall ActionCopyPlotClipbrdExecute(TObject *Sender);
	void __fastcall TrackbarKeyUp(TObject *Sender, WORD &Key,
          TShiftState Shift);
	void __fastcall miStatisticsClick(TObject *Sender);
	void __fastcall ActionShowCommonSettingsExecute(TObject *Sender);
	void __fastcall ActionModeRecorderExecute(TObject *Sender);
	void __fastcall btnRecorderStartClick(TObject *Sender);
	void __fastcall btnRecorderStopClick(TObject *Sender);
	void __fastcall btnRecorderClearClick(TObject *Sender);
	void __fastcall tmrRecorderTimer(TObject *Sender);
	void __fastcall ActionExportPlotBitmapExecute(TObject *Sender);
	void __fastcall ActionCalibrateSensitivityExecute(TObject *Sender);
	void __fastcall ActionExportDataExecute(TObject *Sender);
	void __fastcall FormShow(TObject *Sender);
	void __fastcall ActionAutoFitYExecute(TObject *Sender);
	void __fastcall ActionAutoFitXExecute(TObject *Sender);
	void __fastcall ActionLoadJsonFileExecute(TObject *Sender);
	void __fastcall ActionShowDataFilterChainExecute(TObject *Sender);
	void __fastcall ActionMs3RecordExecute(TObject *Sender);
	void __fastcall ActionMs3LoadExecute(TObject *Sender);
	void __fastcall btnRecordingIntervalApplyClick(TObject *Sender);
	void __fastcall ActionMs3FileRepairExecute(TObject *Sender);

private:	// User declarations
	/** \brief Pointer to plot graph object
	*/
	Display *display;
	/** \brief Type of tool to use with plot graph
	*/
	enum E_MOUSE_TOOL { MOUSE_CURSOR = 0,
						MOUSE_OFFSET,
						MOUSE_ZOOM,
						MOUSE_ZOOM_X,
						MOUSE_ZOOM_Y } mouse_tool;
    /** \brief Resources ID's used as cursors for plot graph
    */
	enum E_CURSOR_RES { CURSOR_HAND = 100,
						CURSOR_ZOOM };
    /** \brief Current plot state (depending on mode)
    */
	struct DISP_STATE {
		float fZoomX;
		float fZoomY;
	};
	struct DISP_STATE displayOsc;	///< display state for oscilloscope mode
	struct DISP_STATE displaySpe;	///< display state for spectrum analyzer mode
	struct DISP_STATE displayRec;	///< display state for slow changing signal recorder

	/** \brief Get structure with current display attributes
	*/
	inline struct DISP_STATE& DisplayState(void) {
		if (display->GetMode() == Display::MODE_OSCILLOSCOPE) {
			return displayOsc;
		} else if (display->GetMode() == Display::MODE_RECORDER) {
			return displayRec;
		} else {
			return displaySpe;
		}
	}

	int iStoredTriggerTypePos;

	/** \brief Plot zoom multiplier for every click with zoom tool
	*/
	const float fZoomChangeGrid;
	void UpdateCursorReadings(enum Display::E_DISPLAY_MODE, int cursorId, float fValX, float fValY);
	/** \brief Update displayed info about X/Y axis zoom
	*/
	void UpdateZoomDisplay(void);
	/** \brief Create TImage class object that holds current plot image
	*/
	std::auto_ptr<Graphics::TBitmap> CreatePlotBitmap(void);
	void CalculateSpectrum(void);
	int iPlotMoveStartX, iPlotMoveStartY;	///< temporary variables used to pass offset vector to display
	bool bMovingPlot;                       ///< are we moving plot right now?
	bool bDataUpdate;						///< flag set when new data is acquired
	bool bPaintUpdate;                      ///< flag set when plot canvas has to be refreshed
	bool bFrameProgressUpdate;				///< flag requesting RX progress bar update
	bool bTriggered;
	bool bFrameFilled;
	bool bJsonViewerMode;					///< set when viewing JSON file content
    float fMs3DivX;
	bool bMs3ViewerMode;					///< set when viewing MS3 file content
	std::vector<float> dataOsc;                ///< data container used in oscilloscope and slow signal recorder modes
	std::vector<float> dataOscTmp;			///< temporary container for samples
	std::vector<float> dataFFT;             ///< data container used in spectrum analyzer mode
	std::vector<float> dataRec;             ///< data container used in slow changing signal recorder mode

	struct S_DATAFRAME current;				///< state at the moment of trigger
	struct S_DATAFRAME upcoming;			///< state updated immediately when GUI state changes
											///< purpose: avoid using VCL code inside callbacks used by device dll 

	AnsiString asDevDllDir;					///< device dll's directory
	DeviceInterface *device;                ///< pointer to object that interfaces with data acquisision hardware
	Mutex mutexDeviceData;                  ///< mutex that prevents hazards with simultaneous data updating by device and data plotting

	void UpdateConnectionState(void);
	/** \brief Update user interface to reflect settings types and ranges offered by device
        \param capabilities ptr to structure describing device properties
	*/
	void SetCapabilities(const struct S_SCOPE_CAPABILITIES* const capabilities);
	/** \brief Change device trigger type and update GUI when trigger type is changed by user
	*/
	void UpdateTriggerType(bool bUpdateDevice = true);
	/** \brief Change device coupling (AC/DC/...) type and update GUI when coupling type is changed by user
	*/
	void UpdateCouplingType(void);
	/** \brief Change size of device acquisition buffer
	*/
	void UpdateBufferSize(void);
	/** \brief Change device sensitivity/gain
	*/
	void UpdateSensitivity(bool bUpdateDevice = true);
	/** \brief Change device sampling rate
	*/
	void UpdateSampling(bool bUpdateDevice = true);
	/** \brief Update device triggering voltage level
	*/
    void UpdateTriggerLevel(void);

	void LoadDevice(AnsiString dllname);

	void UpdateTrackBar(TObject *Sender);
	void __fastcall TrackBarMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
	/** \brief Display dialog when input device is not selected
		(probably first run of application)
	*/
	void FirstRunDialog(void);

	void SetMode(enum Display::E_DISPLAY_MODE);

	std::vector<struct DEV_CALIBRATION> calibration;

	void ApplyCalibration(void);

	/** \brief Report program failure with messagebox and log entry.
		Do nothing if status == 0.
	*/
	void ReportError(int status, const char* const description);

public:		// User declarations
	__fastcall TfrmMain(TComponent* Owner);
	__fastcall ~TfrmMain();
	/** \brief Called when log window is closed
	*/
	void CallbackCloseLogFn(void);
	/** \brief Called by device when new data is available
	*/
	void CallbackDataFn(short*, int);
	/** \brief Called by device when connection state is changed
        \param state current connection state
        \param szMsgText additional text message related to current connection state
	*/
	void CallbackConnectFn(int state, char *szMsgText);
	/** \brief Called by device when triggered
	*/
	void CallbackTriggerFn(void);

	/** \brief Request calibration calculation
		\param fVoltageHigh real voltage at left cursor
		\param offset returned 0V offset
		\param gain returned real sensitivity/declared sensitivity coeff
		\return 0 on success
	*/
	int CallbackCalculateCalibrationFn(float fVoltageHigh,	int* const offset, float* const gain);
	/** \brief Store calibration data to device
	*/
	int CallbackStoreCalibrationFn(const int offset, const float gain);

	void CallbackPlotToolbarClickFn(int tag);
	/** \brief Called when closing JSON viewer
	*/
	void CallbackJsonViewerClose(void);
	/** \brief Called when closing MS3 viewer
	*/
	void CallbackMs3ViewerClose(void);
	/** \brief Called when MS3 viewer updates data
	*/
	void CallbackMs3ViewerDataUpdate(void);
};

//---------------------------------------------------------------------------
extern PACKAGE TfrmMain *frmMain;
//---------------------------------------------------------------------------
#endif
