/** \brief Data recorder (storing single frame)
*/

#ifndef RecorderH
#define RecorderH
//---------------------------------------------------------------------------

#include <string>
#include <vector>
#include <sys/timeb.h>

class Recorder
{
public:
	/** Storage format */
	enum E_FMT {
		FMT_JSON,	///< use JSON
		FMT_CSV		///< use CSV
	};

	/** \brief Store single data vector in file
		\param filename name of file to create or overwrite
		\param format data format for storage
		\param data actual data vector
		\param timestamp time of capturing frame
		\param fDivX sampling period used
		\param coupling coupling used
		\param trigger_type type of trigger used
		\return 0 on success
	*/
	static int Store(const std::string &filename, const enum E_FMT format, const std::vector<float> &data,
		const struct timeb &timestamp, float fDivX, int coupling, int trigger_type);

private:
	static int StoreJson(const std::string &filename, const std::vector<float> &data,
		const struct timeb &timestamp, float fDivX, int coupling, int trigger_type);
	static int StoreCsv(const std::string &filename, const std::vector<float> &data,
		float fDivX);
};

#endif
